// z80patch.cpp
// Revision 8-oct-2006

#include "z80snap.h"

#include <iostream>
#include <fstream>
#include <string>
#include <stdexcept>

using z80snap::z80byte;
using z80snap::Regs;
using z80snap::Header;
using z80snap::Snapshot;

using std::ios;
using std::cout;
using std::cerr;
using std::flush;
using std::ifstream;
using std::ofstream;
using std::string;
using std::runtime_error;

namespace
{

void doit (int argc, char * argv [] )
{
	int argpos= 1;
	if (argpos >= argc)
		throw runtime_error ("No input file");
	string infile (argv [argpos] );
	++argpos;
	if (argpos >= argc)
		throw runtime_error ("No output file");
	string outfile (argv [argpos] );
	++argpos;
	if (argpos >= argc)
		throw runtime_error ("No patch file");
	string patchfile (argv [argpos] );
	++argpos;
	if (argpos >= argc)
		throw runtime_error ("No pacth position");
	char * aux;
	unsigned long pos= strtoul (argv [argpos], & aux, 0);
	if (aux == 0 || * aux != '\0')
		throw runtime_error ("Invalid position");

	ifstream in (infile.c_str (), ios::in | ios::binary );
	if (! in.is_open () )
		throw runtime_error ("Can't open input file");

	Snapshot snap (in);
	const Header & head= snap.getheader ();

	Regs const & regs= head.getregs ();
	cerr << "PC: " << regs.PC << '\n';
	cerr << "SP: " << regs.SP << '\n';
	cerr << "Compressed: " <<
		(head.iscompressed () ? "Yes" : "No") << '\n';
	cerr << "Machine type: ";
	switch (head.gettype () )
	{
	case z80snap::Type16:
		cerr << "Spectrum 16K";
		break;
	case z80snap::Type48:
		cerr << "Spectrum 48K";
		break;
	case z80snap::Type128:
		cerr << "Spectrum 128K, Plus, Plus 2 or Plus 3";
		break;
	default:
		cerr << "Unknown";
	}
	cerr << '\n';

	ifstream patch (patchfile.c_str (), ios::in | ios::binary);
	if (! patch.is_open () )
		throw runtime_error ("Can't open patch file");
	for (;;)
	{
		char c= patch.get ();
		if (! patch)
			break;
		snap.poke (pos++, c);
	}
	if (! patch.eof () )
		throw runtime_error ("Error reading patch file");

	ofstream out (outfile.c_str (), ios::out | ios::binary);
	snap.save (out);
}

} // namespace

int main (int argc, char * * argv)
{
	try
	{
		doit (argc, argv);
	}
	catch (std::exception & e)
	{
		cout << flush;
		cerr << "\nERROR: " << e.what () << '\n';
		return 1;
	}
}


// End of z80patch.cpp
