// z80snap.h
// Revision 8-oct-2006
// (C) 2003 Julian Albo.

#include <iostream>
#include <cstddef>
#include <stdexcept>

namespace z80snap {

class ErrorReadingHeader : public std::exception {
public:
	const char * what () const throw ()
	{ return "Error reading Z80 header"; }
};

typedef unsigned char z80byte;

enum MachineType { TypeUnknown, Type16, Type48, Type128 };

struct Regs {
	unsigned A;
	unsigned F;
	unsigned BC;
	unsigned DE;
	unsigned HL;
	unsigned Ap;
	unsigned Fp;
	unsigned BCp;
	unsigned DEp;
	unsigned HLp;
	unsigned IX;
	unsigned IY;
	unsigned PC;
	unsigned SP;
};

class Header {
public:
	Header (std::istream & is, bool debug= false);

	void save (std::ostream & out);

	bool isoldstyle () const { return oldstyle; }
	bool iscompressed () const { return compressed; }
	MachineType gettype () const { return type; }
	const Regs & getregs () const { return regs; }
private:
	static const std::streamsize headsize= 30;
	z80byte rawhead [headsize];
	z80byte * rawext;

	bool oldstyle;
	bool compressed;
	size_t extlength;
	MachineType type;
	Regs regs;
};

class Snapshot {
public:
	Snapshot (std::istream & is, bool debug= false);
	const Header & getheader () const { return head; }

	void save (std::ostream & out);

	void poke (size_t n, z80byte value);

	z80byte peek (size_t n);
	size_t peek16 (size_t n);
	size_t peeklinenumber (size_t n);
	double peeknumber (size_t n);

	const z80byte * basememory () const { return memory; }
	z80byte * basememory () { return memory; }
private:
	Header head;
	void loadblock (std::istream & is,
		z80byte * ptr, std::streamsize length, bool debug);

	size_t compressblock (z80byte * ptr, z80byte * compr);
	void saveblock (std::ostream & out, z80byte * ptr);

	void loadblockv1 (std::istream & is, bool debug);
	z80byte memory [64 * 1024 + 1];
	// Extra byte to simplify end marker detection on v1 block.
	z80byte extramemory [80 * 1024];
};

} // namespace z80snap

// End of z80snap.h
